@@ -1,15 +1,13 @@
-/***********************************************************
-* Rocks'n'Diamonds -- McDuffin Strikes Back!               *
-*----------------------------------------------------------*
-* (c) 1995-2006 Artsoft Entertainment                      *
-*               Holger Schemel                             *
-*               Detmolder Strasse 189                      *
-*               33604 Bielefeld                            *
-*               Germany                                    *
-*               e-mail: info@artsoft.org                   *
-*----------------------------------------------------------*
-* files.h                                                  *
-***********************************************************/
+// ============================================================================
+// Rocks'n'Diamonds - McDuffin Strikes Back!
+// ----------------------------------------------------------------------------
+// (c) 1995-2014 by Artsoft Entertainment
+//     		    Holger Schemel
+//		    info@artsoft.org
+//		    http://www.artsoft.org/
+// ----------------------------------------------------------------------------
+// files.h
+// ============================================================================
 
 #ifndef FILES_H
 #define FILES_H
@@ -32,6 +30,8 @@ void setElementChangeInfoToDefaults(struct ElementChangeInfo *);
 void copyElementInfo(struct ElementInfo *, struct ElementInfo *);
 
 char *getDefaultLevelFilename(int);
+char *getLocalLevelTemplateFilename();
+char *getGlobalLevelTemplateFilename();
 
 void LoadLevelFromFilename(struct LevelInfo *, char *);
 void LoadLevel(int);
@@ -56,6 +56,7 @@ boolean SaveTapeChecked(int);
 void LoadScore(int);
 void SaveScore(int);
 
+void LoadSetupFromFilename(char *);
 void LoadSetup();
 void SaveSetup();
 
@@ -65,6 +66,7 @@ void SaveSetup_EditorCascade();
 void LoadCustomElementDescriptions();
 void InitMenuDesignSettings_Static();
 void LoadMenuDesignSettings();
+void LoadMenuDesignSettings_AfterGraphics();
 void LoadUserDefinedEditorElementList(int **, int *);
 void LoadMusicInfo();
 void LoadHelpAnimInfo();
@@ -72,5 +74,6 @@ void LoadHelpTextInfo();
 
 void ConvertLevels();
 void CreateLevelSketchImages();
+void CreateCustomElementImages(char *);
 
 #endif	/* FILES_H */
