@@ -1,15 +1,13 @@
-/***********************************************************
-* Artsoft Retro-Game Library                               *
-*----------------------------------------------------------*
-* (c) 1994-2006 Artsoft Entertainment                      *
-*               Holger Schemel                             *
-*               Detmolder Strasse 189                      *
-*               33604 Bielefeld                            *
-*               Germany                                    *
-*               e-mail: info@artsoft.org                   *
-*----------------------------------------------------------*
-* system.h                                                 *
-***********************************************************/
+// ============================================================================
+// Artsoft Retro-Game Library
+// ----------------------------------------------------------------------------
+// (c) 1995-2014 by Artsoft Entertainment
+//     		    Holger Schemel
+//		    info@artsoft.org
+//		    http://www.artsoft.org/
+// ----------------------------------------------------------------------------
+// system.h
+// ============================================================================
 
 #ifndef SYSTEM_H
 #define SYSTEM_H
@@ -22,15 +20,11 @@
 #include "macosx.h"
 #elif defined(PLATFORM_WIN32)
 #include "windows.h"
-#elif defined(PLATFORM_MSDOS)
-#include "msdos.h"
+#elif defined(PLATFORM_ANDROID)
+#include "android.h"
 #endif
 
-#if defined(TARGET_SDL)
 #include "sdl.h"
-#elif defined(TARGET_X11)
-#include "x11.h"
-#endif
 
 
 /* the additional 'b' is needed for Win32 to open files in binary mode */
@@ -45,9 +39,69 @@
 #define BLIT_INVERSE			2
 #define BLIT_ON_BACKGROUND		3
 
+/* values for fullscreen status */
 #define FULLSCREEN_NOT_AVAILABLE	FALSE
 #define FULLSCREEN_AVAILABLE		TRUE
 
+/* values for window scaling */
+#define WINDOW_SCALING_NOT_AVAILABLE	FALSE
+#define WINDOW_SCALING_AVAILABLE	TRUE
+
+#define MIN_WINDOW_SCALING_PERCENT	30
+#define STD_WINDOW_SCALING_PERCENT	100
+#define MAX_WINDOW_SCALING_PERCENT	400
+#define STEP_WINDOW_SCALING_PERCENT	10
+
+/* values for window scaling quality */
+#define SCALING_QUALITY_NEAREST		"nearest"
+#define SCALING_QUALITY_LINEAR		"linear"
+#define SCALING_QUALITY_BEST		"best"
+
+#define SCALING_QUALITY_DEFAULT		SCALING_QUALITY_LINEAR
+
+/* values for screen rendering mode */
+#define STR_SPECIAL_RENDERING_OFF	"stream_texture_only"
+#define STR_SPECIAL_RENDERING_BITMAP	"bitmap_and_stream_texture"
+#define STR_SPECIAL_RENDERING_TARGET	"target_texture_only"
+#define STR_SPECIAL_RENDERING_DOUBLE	"stream_and_target_texture"
+
+#if defined(TARGET_SDL2)
+#define STR_SPECIAL_RENDERING_DEFAULT	STR_SPECIAL_RENDERING_DOUBLE
+#else
+#define STR_SPECIAL_RENDERING_DEFAULT	STR_SPECIAL_RENDERING_BITMAP
+#endif
+
+#define SPECIAL_RENDERING_OFF		0
+#define SPECIAL_RENDERING_BITMAP	1
+#define SPECIAL_RENDERING_TARGET	2
+#define SPECIAL_RENDERING_DOUBLE	3
+
+#if defined(TARGET_SDL2)
+#define SPECIAL_RENDERING_DEFAULT	SPECIAL_RENDERING_DOUBLE
+#else
+#define SPECIAL_RENDERING_DEFAULT	SPECIAL_RENDERING_BITMAP
+#endif
+
+/* values for touch control */
+#define TOUCH_CONTROL_VIRTUAL_BUTTONS	"virtual_buttons"
+#define TOUCH_CONTROL_WIPE_GESTURES	"wipe_gestures"
+#define TOUCH_CONTROL_FOLLOW_FINGER	"follow_finger"
+
+#define TOUCH_CONTROL_DEFAULT		TOUCH_CONTROL_VIRTUAL_BUTTONS
+
+#define TOUCH_MOVE_DISTANCE_DEFAULT	2
+#define TOUCH_DROP_DISTANCE_DEFAULT	5
+
+
+/* values for special settings for mobile devices */
+#if defined(PLATFORM_ANDROID)
+#define USE_TOUCH_INPUT_OVERLAY
+#define USE_COMPLETE_DISPLAY
+#define HAS_SCREEN_KEYBOARD
+#define SCREEN_KEYBOARD_POS(h)		((h) / 2)
+#endif
+
+
 /* default input keys */
 #define DEFAULT_KEY_LEFT		KSYM_Left
 #define DEFAULT_KEY_RIGHT		KSYM_Right
@@ -86,6 +140,34 @@
 #define DEFAULT_KEY_SNAP_UP		KSYM_UNDEFINED
 #define DEFAULT_KEY_SNAP_DOWN		KSYM_UNDEFINED
 
+/* default debug setup keys and values */
+#define DEFAULT_FRAME_DELAY_0		20		// 100 % speed
+#define DEFAULT_FRAME_DELAY_1		500		// 4 % speed
+#define DEFAULT_FRAME_DELAY_2		250		// 8 % speed
+#define DEFAULT_FRAME_DELAY_3		125		// 16 % speed
+#define DEFAULT_FRAME_DELAY_4		60		// 33 % speed
+#define DEFAULT_FRAME_DELAY_5		40		// 50 % speed
+#define DEFAULT_FRAME_DELAY_6		30		// 66 % speed
+#define DEFAULT_FRAME_DELAY_7		10		// 200 % speed
+#define DEFAULT_FRAME_DELAY_8		5		// 400 % speed
+#define DEFAULT_FRAME_DELAY_9		0		// maximum speed
+
+#define DEFAULT_KEY_FRAME_DELAY_0	KSYM_0
+#define DEFAULT_KEY_FRAME_DELAY_1	KSYM_1
+#define DEFAULT_KEY_FRAME_DELAY_2	KSYM_2
+#define DEFAULT_KEY_FRAME_DELAY_3	KSYM_3
+#define DEFAULT_KEY_FRAME_DELAY_4	KSYM_4
+#define DEFAULT_KEY_FRAME_DELAY_5	KSYM_5
+#define DEFAULT_KEY_FRAME_DELAY_6	KSYM_6
+#define DEFAULT_KEY_FRAME_DELAY_7	KSYM_7
+#define DEFAULT_KEY_FRAME_DELAY_8	KSYM_8
+#define DEFAULT_KEY_FRAME_DELAY_9	KSYM_9
+
+#define NUM_DEBUG_FRAME_DELAY_KEYS	10
+
+#define DEFAULT_FRAME_DELAY_USE_MOD_KEY	FALSE
+#define DEFAULT_FRAME_DELAY_GAME_ONLY	TRUE
+
 /* values for key_status */
 #define KEY_NOT_PRESSED			FALSE
 #define KEY_RELEASED			FALSE
@@ -107,14 +189,18 @@
 #define MB_WHEEL_DOWN			5
 #define MB_WHEEL_LEFT			6
 #define MB_WHEEL_RIGHT			7
-#define IS_WHEEL_BUTTON_VERTICAL(b)	((b) >= MB_WHEEL_UP &&		\
-					 (b) <= MB_WHEEL_DOWN)
-#define IS_WHEEL_BUTTON_HORIZONTAL(b)	((b) >= MB_WHEEL_LEFT &&	\
-					 (b) <= MB_WHEEL_RIGHT)
-#define IS_WHEEL_BUTTON(b)		((b) >= MB_WHEEL_UP &&		\
-					 (b) <= MB_WHEEL_DOWN)
+#define IS_WHEEL_BUTTON_VERTICAL(b)	((b) == MB_WHEEL_UP ||		\
+					 (b) == MB_WHEEL_DOWN)
+#define IS_WHEEL_BUTTON_HORIZONTAL(b)	((b) == MB_WHEEL_LEFT ||	\
+					 (b) == MB_WHEEL_RIGHT)
+#define IS_WHEEL_BUTTON(b)		(IS_WHEEL_BUTTON_VERTICAL(b) ||	\
+					 IS_WHEEL_BUTTON_HORIZONTAL(b))
 #define DEFAULT_WHEEL_STEPS		3
 
+#define BUTTON_STEPSIZE(b)		((b) == MB_LEFTBUTTON   ?  1 :	\
+					 (b) == MB_MIDDLEBUTTON ?  5 :	\
+					 (b) == MB_RIGHTBUTTON  ? 10 : 1)
+
 /* values for move directions */
 #define MV_BIT_LEFT			0
 #define MV_BIT_RIGHT			1
@@ -140,6 +226,17 @@
 /* values for special "focus player" bitmasks */
 #define BIT_SET_FOCUS			6
 
+/* values for drawing stages for global animations */
+#define DRAW_GLOBAL_ANIM_STAGE_1	1
+#define DRAW_GLOBAL_ANIM_STAGE_2	2
+
+/* values for drawing target (various functions) */
+#define DRAW_TO_BACKBUFFER		0
+#define DRAW_TO_FIELDBUFFER		1
+#define DRAW_TO_SCREEN			2
+#define DRAW_TO_FADE_SOURCE		3
+#define DRAW_TO_FADE_TARGET		4
+
 /* values for move directions and special "button" key bitmasks */
 #define MV_NONE			0
 #define MV_LEFT			(1 << MV_BIT_LEFT)
@@ -196,6 +293,7 @@
 				 MV_NONE)
 
 /* values for animation mode (frame order and direction) */
+/* (stored in level files -- never change existing values) */
 #define ANIM_NONE		0
 #define ANIM_LOOP		(1 << 0)
 #define ANIM_LINEAR		(1 << 1)
@@ -209,10 +307,13 @@
 #define ANIM_OPAQUE_PLAYER	(1 << 9)
 
 /* values for special (non game element) animation modes */
+/* (not stored in level files -- can be changed, if needed) */
 #define ANIM_HORIZONTAL		(1 << 10)
 #define ANIM_VERTICAL		(1 << 11)
 #define ANIM_CENTERED		(1 << 12)
 #define ANIM_STATIC_PANEL	(1 << 13)
+#define ANIM_ALL		(1 << 14)
+#define ANIM_ONCE		(1 << 15)
 
 #define ANIM_DEFAULT		ANIM_LOOP
 
@@ -230,7 +331,8 @@
 #define FADE_TYPE_TRANSFORM	(1 << 2)
 #define FADE_TYPE_CROSSFADE	(1 << 3)
 #define FADE_TYPE_MELT		(1 << 4)
-#define FADE_TYPE_SKIP		(1 << 5)
+#define FADE_TYPE_CURTAIN	(1 << 5)
+#define FADE_TYPE_SKIP		(1 << 6)
 
 #define FADE_MODE_NONE		(FADE_TYPE_NONE)
 #define FADE_MODE_FADE_IN	(FADE_TYPE_FADE_IN)
@@ -239,11 +341,23 @@
 #define FADE_MODE_TRANSFORM	(FADE_TYPE_TRANSFORM | FADE_TYPE_FADE_IN)
 #define FADE_MODE_CROSSFADE	(FADE_MODE_TRANSFORM | FADE_TYPE_CROSSFADE)
 #define FADE_MODE_MELT		(FADE_MODE_TRANSFORM | FADE_TYPE_MELT)
+#define FADE_MODE_CURTAIN	(FADE_MODE_TRANSFORM | FADE_TYPE_CURTAIN)
 #define FADE_MODE_SKIP_FADE_IN	(FADE_TYPE_SKIP | FADE_TYPE_FADE_IN)
 #define FADE_MODE_SKIP_FADE_OUT	(FADE_TYPE_SKIP | FADE_TYPE_FADE_OUT)
 
 #define FADE_MODE_DEFAULT	FADE_MODE_FADE
 
+/* values for toon positions */
+#define POS_UNDEFINED		-1
+#define POS_LEFT		0
+#define POS_RIGHT		1
+#define POS_TOP			2
+#define POS_UPPER		3
+#define POS_MIDDLE		4
+#define POS_LOWER		5
+#define POS_BOTTOM		6
+#define POS_ANY			7
+
 /* values for text alignment */
 #define ALIGN_LEFT		(1 << 0)
 #define ALIGN_RIGHT		(1 << 1)
@@ -266,44 +380,27 @@
 #define REDRAW_NONE		(0)
 #define REDRAW_ALL		(1 << 0)
 #define REDRAW_FIELD		(1 << 1)
-#define REDRAW_TILES		(1 << 2)
-#define REDRAW_DOOR_1		(1 << 3)
-#define REDRAW_VIDEO_1		(1 << 4)
-#define REDRAW_VIDEO_2		(1 << 5)
-#define REDRAW_VIDEO_3		(1 << 6)
-#define REDRAW_MICROLEVEL	(1 << 7)
-#define REDRAW_MICROLABEL	(1 << 8)
-#define REDRAW_FROM_BACKBUFFER	(1 << 9)
-#define REDRAW_DOOR_2		(REDRAW_VIDEO_1 | \
-				 REDRAW_VIDEO_2 | \
-				 REDRAW_VIDEO_3)
-#define REDRAW_DOOR_3		(1 << 10)
+#define REDRAW_DOOR_1		(1 << 2)
+#define REDRAW_DOOR_2		(1 << 3)
+#define REDRAW_DOOR_3		(1 << 4)
+#define REDRAW_FPS		(1 << 5)
+
 #define REDRAW_DOORS		(REDRAW_DOOR_1 | \
 				 REDRAW_DOOR_2 | \
 				 REDRAW_DOOR_3)
-#define REDRAW_MAIN		(REDRAW_FIELD | \
-				 REDRAW_TILES | \
-				 REDRAW_MICROLEVEL)
-#define REDRAW_FPS		(1 << 11)
 
-#if defined(TARGET_X11)
-/* on old-style, classic and potentially slow graphics systems, redraw single
-   tiles instead of the whole playfield unless a certain threshold is reached;
-   when using the X11 target, this method should still be fast on all systems */
-#define REDRAWTILES_THRESHOLD	(SCR_FIELDX * SCR_FIELDY / 2)
-#else
-/* on modern graphics systems and when using the SDL target, this tile redraw
-   optimization can slow things down a lot due to many small blits compared to
-   one single playfield-sized blit (especially observed on Mac OS X with SDL) */
-#define REDRAWTILES_THRESHOLD	0
-#endif
-
-#define IN_GFX_SCREEN(x, y)	(x >= gfx.sx && x < gfx.sx + gfx.sxsize && \
+#define IN_GFX_FIELD_PLAY(x, y)	(x >= gfx.sx && x < gfx.sx + gfx.sxsize && \
 				 y >= gfx.sy && y < gfx.sy + gfx.sysize)
-#define IN_GFX_DOOR(x, y)	(x >= gfx.dx && x < gfx.dx + gfx.dxsize && \
+#define IN_GFX_FIELD_FULL(x, y)	(x >= gfx.real_sx && \
+				 x <  gfx.real_sx + gfx.full_sxsize && \
+				 y >= gfx.real_sy && \
+				 y <  gfx.real_sy + gfx.full_sysize)
+#define IN_GFX_DOOR_1(x, y)	(x >= gfx.dx && x < gfx.dx + gfx.dxsize && \
 				 y >= gfx.dy && y < gfx.dy + gfx.dysize)
-#define IN_GFX_VIDEO(x, y)	(x >= gfx.vx && x < gfx.vx + gfx.vxsize && \
+#define IN_GFX_DOOR_2(x, y)	(x >= gfx.vx && x < gfx.vx + gfx.vxsize && \
 				 y >= gfx.vy && y < gfx.vy + gfx.vysize)
+#define IN_GFX_DOOR_3(x, y)	(x >= gfx.ex && x < gfx.ex + gfx.exsize && \
+				 y >= gfx.ey && y < gfx.ey + gfx.eysize)
 
 /* values for mouse cursor */
 #define CURSOR_DEFAULT		0
@@ -312,6 +409,7 @@
 
 /* fundamental game speed values */
 #define ONE_SECOND_DELAY	1000	/* delay value for one second */
+#define MENU_FRAME_DELAY	20	/* frame delay in milliseconds */
 #define GAME_FRAME_DELAY	20	/* frame delay in milliseconds */
 #define FFWD_FRAME_DELAY	10	/* 200% speed for fast forward */
 #define FRAMES_PER_SECOND	(ONE_SECOND_DELAY / GAME_FRAME_DELAY)
@@ -348,12 +446,18 @@
 /* default value for undefined filename */
 #define UNDEFINED_FILENAME	"[NONE]"
 
+/* default value for undefined levelset */
+#define UNDEFINED_LEVELSET	"[NONE]"
+
 /* default value for undefined parameter */
 #define ARG_DEFAULT		"[DEFAULT]"
 
 /* default values for undefined configuration file parameters */
 #define ARG_UNDEFINED		"-1000000"
-#define ARG_UNDEFINED_VALUE	(atoi(ARG_UNDEFINED))
+#define ARG_UNDEFINED_VALUE	(-1000000)
+
+/* default value for off-screen positions */
+#define POS_OFFSCREEN		(-1000000)
 
 /* definitions for game sub-directories */
 #ifndef RO_GAME_DIR
@@ -377,34 +481,21 @@
 #define DOCS_DIRECTORY		"docs"
 #define CACHE_DIRECTORY		"cache"
 
-#if !defined(PLATFORM_MSDOS)
 #define GFX_CLASSIC_SUBDIR	"gfx_classic"
 #define SND_CLASSIC_SUBDIR	"snd_classic"
 #define MUS_CLASSIC_SUBDIR	"mus_classic"
-#else
-#define GFX_CLASSIC_SUBDIR	"gfx_orig"
-#define SND_CLASSIC_SUBDIR	"snd_orig"
-#define MUS_CLASSIC_SUBDIR	"mus_orig"
-#endif
 
-#if defined(CREATE_SPECIAL_EDITION_RND_JUE)
-#define GFX_DEFAULT_SUBDIR	"jue0"
-#define SND_DEFAULT_SUBDIR	"jue0"
-#define MUS_DEFAULT_SUBDIR	"jue0"
-#else
-#define GFX_DEFAULT_SUBDIR	GFX_CLASSIC_SUBDIR
-#define SND_DEFAULT_SUBDIR	SND_CLASSIC_SUBDIR
-#define MUS_DEFAULT_SUBDIR	MUS_CLASSIC_SUBDIR
-#endif
+#define GFX_DEFAULT_SUBDIR	(setup.internal.default_graphics_set)
+#define SND_DEFAULT_SUBDIR	(setup.internal.default_sounds_set)
+#define MUS_DEFAULT_SUBDIR	(setup.internal.default_music_set)
 
-#if defined(CREATE_SPECIAL_EDITION)
-#define GFX_FALLBACK_FILENAME	"fallback.pcx"
-#define SND_FALLBACK_FILENAME	"fallback.wav"
-#define MUS_FALLBACK_FILENAME	"fallback.wav"
-#endif
+#define GFX_FALLBACK_FILENAME	(setup.internal.fallback_graphics_file)
+#define SND_FALLBACK_FILENAME	(setup.internal.fallback_sounds_file)
+#define MUS_FALLBACK_FILENAME	(setup.internal.fallback_music_file)
+
+#define DEFAULT_LEVELSET	(setup.internal.default_level_series)
 
 /* file names and filename extensions */
-#if !defined(PLATFORM_MSDOS)
 #define LEVELSETUP_DIRECTORY	"levelsetup"
 #define SETUP_FILENAME		"setup.conf"
 #define LEVELSETUP_FILENAME	"levelsetup.conf"
@@ -417,28 +508,20 @@
 #define SOUNDSINFO_FILENAME	"soundsinfo.conf"
 #define MUSICINFO_FILENAME	"musicinfo.conf"
 #define ARTWORKINFO_CACHE_FILE	"artworkinfo.cache"
+#define LEVELTEMPLATE_FILENAME	"template.level"
 #define LEVELFILE_EXTENSION	"level"
 #define TAPEFILE_EXTENSION	"tape"
 #define SCOREFILE_EXTENSION	"score"
-#else
-#define LEVELSETUP_DIRECTORY	"lvlsetup"
-#define SETUP_FILENAME		"setup.cnf"
-#define LEVELSETUP_FILENAME	"lvlsetup.cnf"
-#define EDITORSETUP_FILENAME	"edsetup.cnf"
-#define EDITORCASCADE_FILENAME	"edcascad.conf"
-#define HELPANIM_FILENAME	"helpanim.cnf"
-#define HELPTEXT_FILENAME	"helptext.cnf"
-#define LEVELINFO_FILENAME	"lvlinfo.cnf"
-#define GRAPHICSINFO_FILENAME	"gfxinfo.cnf"
-#define SOUNDSINFO_FILENAME	"sndinfo.cnf"
-#define MUSICINFO_FILENAME	"musinfo.cnf"
-#define ARTWORKINFO_CACHE_FILE	"artinfo.cac"
-#define LEVELFILE_EXTENSION	"lvl"
-#define TAPEFILE_EXTENSION	"tap"
-#define SCOREFILE_EXTENSION	"sco"
-#endif
 
-#define ERROR_BASENAME		"stderr.txt"
+#define LOG_OUT_BASENAME	"stdout.txt"
+#define LOG_ERR_BASENAME	"stderr.txt"
+
+#define LOG_OUT_ID		0
+#define LOG_ERR_ID		1
+#define NUM_LOGS		2
+
+#define STRING_PARENT_DIRECTORY		".."
+#define STRING_TOP_DIRECTORY		"/"
 
 #define CHAR_PATH_SEPARATOR_UNIX	'/'
 #define CHAR_PATH_SEPARATOR_DOS		'\\'
@@ -449,7 +532,7 @@
 #define STRING_NEWLINE_UNIX		"\n"
 #define STRING_NEWLINE_DOS		"\r\n"
 
-#if defined(PLATFORM_WIN32) || defined(PLATFORM_MSDOS)
+#if defined(PLATFORM_WIN32)
 #define CHAR_PATH_SEPARATOR	CHAR_PATH_SEPARATOR_DOS
 #define STRING_PATH_SEPARATOR	STRING_PATH_SEPARATOR_DOS
 #define STRING_NEWLINE		STRING_NEWLINE_DOS
@@ -643,10 +726,6 @@
 }
 
 
-/* type definitions */
-typedef int (*EventFilter)(const Event *);
-
-
 /* structure definitions */
 
 struct ProgramInfo
@@ -654,38 +733,43 @@ struct ProgramInfo
   char *command_basepath;	/* path to the program binary */
   char *command_basename;	/* base filename of the program binary */
 
+  char *config_filename;	/* optional global program config filename */
+
   char *maindata_path;		/* main game data (installation) directory */
 
   char *userdata_subdir;	/* personal user game data directory */
-  char *userdata_subdir_unix;	/* personal user game data directory (Unix) */
   char *userdata_path;		/* resulting full path to game data directory */
 
   char *program_title;
   char *window_title;
   char *icon_title;
 
-  char *x11_icon_filename;
-  char *x11_iconmask_filename;
-  char *sdl_icon_filename;
-  char *msdos_cursor_filename;
+  char *icon_filename;
 
   char *cookie_prefix;
-  char *filename_prefix;	/* prefix to cut off from DOS filenames */
 
-  char *error_filename;		/* filename where to write error messages to */
-  FILE *error_file;		/* (used instead of 'stderr' on some systems) */
+  char *log_filename[NUM_LOGS];		/* log filenames for out/err messages */
+  FILE *log_file[NUM_LOGS];		/* log file handles for out/err files */
+  FILE *log_file_default[NUM_LOGS];	/* default log file handles (out/err) */
 
   int version_major;
   int version_minor;
   int version_patch;
+  int version_build;
+  int version_ident;
 
+  char *(*window_title_function)(void);
   void (*exit_message_function)(char *, va_list);
   void (*exit_function)(int);
+
+  boolean global_scores;
+  boolean many_scores_per_name;
+
+  boolean headless;
 };
 
 struct OptionInfo
 {
-  char *display_name;
   char *server_host;
   int server_port;
 
@@ -701,27 +785,41 @@ struct OptionInfo
 
   char *special_flags;
 
+  boolean mytapes;
   boolean serveronly;
   boolean network;
   boolean verbose;
   boolean debug;
-  boolean debug_x11_sync;
-};
-
-struct ScreenModeInfo
-{
-  int width, height;
 };
 
 struct VideoSystemInfo
 {
   int default_depth;
   int width, height, depth;
+  int window_width, window_height;
+  int display_width, display_height;
+  int screen_width, screen_height;
+  int screen_xoffset, screen_yoffset;
 
   boolean fullscreen_available;
   boolean fullscreen_enabled;
-  struct ScreenModeInfo *fullscreen_modes;
-  char *fullscreen_mode_current;
+  boolean fullscreen_initial;
+
+  boolean window_scaling_available;
+  int window_scaling_percent;
+  char *window_scaling_quality;
+  int screen_rendering_mode;
+
+  unsigned int frame_delay;
+  unsigned int frame_delay_value;
+
+  boolean shifted_up;
+  int shifted_up_pos;
+  int shifted_up_pos_last;
+  unsigned int shifted_up_delay;
+  unsigned int shifted_up_delay_value;
+
+  boolean initialized;
 };
 
 struct AudioSystemInfo
@@ -755,10 +853,6 @@ struct FontBitmapInfo
 
   int num_chars;
   int num_chars_per_line;
-
-#if defined(TARGET_X11_NATIVE_PERFORMANCE_WORKAROUND)
-  Pixmap *clip_mask;		/* single-char-only clip mask array for X11 */
-#endif
 };
 
 struct GfxInfo
@@ -769,12 +863,17 @@ struct GfxInfo
   int full_sxsize, full_sysize;
   int scrollbuffer_width, scrollbuffer_height;
 
+  int game_tile_size, standard_tile_size;
+
   int dx, dy;
   int dxsize, dysize;
 
   int vx, vy;
   int vxsize, vysize;
 
+  int ex, ey;
+  int exsize, eysize;
+
   int win_xsize, win_ysize;
 
   int draw_deactivation_mask;
@@ -785,6 +884,17 @@ struct GfxInfo
   Bitmap *background_bitmap;
   int background_bitmap_mask;
 
+  Bitmap *fade_bitmap_backup;
+  Bitmap *fade_bitmap_source;
+  Bitmap *fade_bitmap_target;
+  Bitmap *fade_bitmap_black;
+
+  int fade_border_source_status;
+  int fade_border_target_status;
+  Bitmap *masked_border_bitmap_ptr;
+
+  Bitmap *final_screen_bitmap;
+
   boolean clipping_enabled;
   int clip_x, clip_y;
   int clip_width, clip_height;
@@ -803,6 +913,15 @@ struct GfxInfo
   int anim_random_frame;
 
   void (*draw_busy_anim_function)(void);
+  void (*draw_global_anim_function)(int, int);
+  void (*draw_global_border_function)(int);
+
+  int cursor_mode;
+};
+
+struct OverlayInfo
+{
+  boolean active;
 };
 
 struct JoystickInfo
@@ -826,6 +945,13 @@ struct SetupKeyboardInfo
   Key snap, drop;
 };
 
+struct SetupTouchInfo
+{
+  char *control_type;
+  int move_distance;
+  int drop_distance;
+};
+
 struct SetupInputInfo
 {
   boolean use_joystick;
@@ -845,6 +971,8 @@ struct SetupEditorInfo
   boolean el_dx_boulderdash;
   boolean el_chars;
   boolean el_steel_chars;
+
+  boolean el_classic;
   boolean el_custom;
   boolean el_user_defined;
   boolean el_dynamic;
@@ -855,6 +983,8 @@ struct SetupEditorInfo
   boolean el_by_type;
 
   boolean show_element_token;
+
+  boolean use_template_for_new_levels;
 };
 
 struct SetupEditorCascadeInfo
@@ -909,6 +1039,41 @@ struct SetupSystemInfo
   int audio_fragment_size;
 };
 
+struct SetupInternalInfo
+{
+  char *program_title;
+  char *program_author;
+  char *program_email;
+  char *program_website;
+  char *program_copyright;
+  char *program_company;
+
+  char *program_icon_file;
+
+  char *default_graphics_set;
+  char *default_sounds_set;
+  char *default_music_set;
+
+  char *fallback_graphics_file;
+  char *fallback_sounds_file;
+  char *fallback_music_file;
+
+  char *default_level_series;
+
+  int default_window_width;
+  int default_window_height;
+
+  boolean choose_from_top_leveldir;
+};
+
+struct SetupDebugInfo
+{
+  int frame_delay[10];
+  Key frame_delay_key[10];
+  boolean frame_delay_use_mod_key;
+  boolean frame_delay_game_only;
+};
+
 struct SetupInfo
 {
   char *player_name;
@@ -920,7 +1085,8 @@ struct SetupInfo
   boolean toons;
   boolean scroll_delay;
   boolean scroll_delay_value;
-  boolean soft_scrolling;
+  char *engine_snapshot_mode;
+  int engine_snapshot_memory;
   boolean fade_screens;
   boolean autorecord;
   boolean show_titlescreen;
@@ -928,9 +1094,12 @@ struct SetupInfo
   boolean team_mode;
   boolean handicap;
   boolean skip_levels;
+  boolean increment_levels;
   boolean time_limit;
   boolean fullscreen;
-  char *fullscreen_mode;
+  int window_scaling_percent;
+  char *window_scaling_quality;
+  char *screen_rendering_mode;
   boolean ask_on_escape;
   boolean ask_on_escape_editor;
   boolean quick_switch;
@@ -939,6 +1108,7 @@ struct SetupInfo
   int game_frame_delay;
   boolean sp_show_border_elements;
   boolean small_game_graphics;
+  boolean show_snapshot_buttons;
 
   char *graphics_set;
   char *sounds_set;
@@ -955,7 +1125,11 @@ struct SetupInfo
   struct SetupEditorCascadeInfo editor_cascade;
   struct SetupShortcutInfo shortcut;
   struct SetupInputInfo input[MAX_PLAYERS];
+  struct SetupTouchInfo touch;
   struct SetupSystemInfo system;
+  struct SetupInternalInfo internal;
+  struct SetupDebugInfo debug;
+
   struct OptionInfo options;
 };
 
@@ -1148,6 +1322,12 @@ struct XY
   int x, y;
 };
 
+struct XYTileSize
+{
+  int x, y;
+  int tile_size;
+};
+
 struct Rect
 {
   int x, y;
@@ -1168,15 +1348,31 @@ struct MenuPosInfo
   int align, valign;
 };
 
+struct DoorPartPosInfo
+{
+  int x, y;
+  int step_xoffset;
+  int step_yoffset;
+  int step_delay;
+  int start_step;
+  int start_step_opening;
+  int start_step_closing;
+  boolean draw_masked;
+  int sort_priority;
+};
+
 struct TextPosInfo
 {
   int x, y;
+  int xoffset;			/* special case for tape date and time */
+  int xoffset2;			/* special case for tape date */
   int width, height;
   int align, valign;
-  int size;
+  int size;			/* also used for suffix ".digits" */
   int font, font_alt;
   boolean draw_masked;
-  int sort_priority;
+  boolean draw_player;		/* special case for network player buttons */
+  int sort_priority;		/* also used for suffix ".draw_order" */
   int id;
 };
 
@@ -1196,6 +1392,7 @@ extern struct OptionInfo	options;
 extern struct VideoSystemInfo	video;
 extern struct AudioSystemInfo	audio;
 extern struct GfxInfo		gfx;
+extern struct OverlayInfo	overlay;
 extern struct AnimInfo		anim;
 extern struct ArtworkInfo	artwork;
 extern struct JoystickInfo	joystick;
@@ -1208,20 +1405,18 @@ extern int			level_nr;
 
 extern struct LevelStats	level_stats[];
 
-extern Display		       *display;
-extern Visual		       *visual;
-extern int			screen;
-extern Colormap			cmap;
-
 extern DrawWindow	       *window;
 extern DrawBuffer	       *backbuffer;
 extern DrawBuffer	       *drawto;
 
 extern int			button_status;
 extern boolean			motion_status;
+extern int			wheel_steps;
+#if defined(TARGET_SDL2)
+extern boolean			keyrepeat_status;
+#endif
 
 extern int			redraw_mask;
-extern int			redraw_tiles;
 
 extern int			FrameCounter;
 
@@ -1229,46 +1424,65 @@ extern int			FrameCounter;
 /* function definitions */
 
 void InitProgramInfo(char *, char *, char *, char *, char *, char *, char *,
-		     char *, char *, char *, char *, char *, int);
+		     int);
 
+void InitScoresInfo();
+void SetWindowTitle();
+
+void InitWindowTitleFunction(char *(*window_title_function)(void));
 void InitExitMessageFunction(void (*exit_message_function)(char *, va_list));
 void InitExitFunction(void (*exit_function)(int));
 void InitPlatformDependentStuff(void);
 void ClosePlatformDependentStuff(void);
 
 void InitGfxFieldInfo(int, int, int, int, int, int, int, int, Bitmap *);
+void InitGfxTileSizeInfo(int, int);
 void InitGfxDoor1Info(int, int, int, int);
 void InitGfxDoor2Info(int, int, int, int);
+void InitGfxDoor3Info(int, int, int, int);
 void InitGfxWindowInfo(int, int);
 void InitGfxScrollbufferInfo(int, int);
 void InitGfxClipRegion(boolean, int, int, int, int);
 void InitGfxDrawBusyAnimFunction(void (*draw_busy_anim_function)(void));
+void InitGfxDrawGlobalAnimFunction(void (*draw_global_anim_function)(int, int));
+void InitGfxDrawGlobalBorderFunction(void (*draw_global_border_function)(int));
 void InitGfxCustomArtworkInfo();
+void InitGfxOtherSettings();
+void InitOverlayInfo();
+void SetOverlayActive(boolean);
+boolean GetOverlayActive();
 void SetDrawDeactivationMask(int);
 void SetDrawBackgroundMask(int);
 void SetWindowBackgroundBitmap(Bitmap *);
 void SetMainBackgroundBitmap(Bitmap *);
 void SetDoorBackgroundBitmap(Bitmap *);
+void SetRedrawMaskFromArea(int, int, int, int);
+
+void LimitScreenUpdates(boolean);
 
 void InitVideoDisplay(void);
 void CloseVideoDisplay(void);
 void InitVideoBuffer(int, int, int, boolean);
 Bitmap *CreateBitmapStruct(void);
 Bitmap *CreateBitmap(int, int, int);
-void ReCreateBitmap(Bitmap **, int, int, int);
+void ReCreateBitmap(Bitmap **, int, int);
 void FreeBitmap(Bitmap *);
 void BlitBitmap(Bitmap *, Bitmap *, int, int, int, int, int, int);
 void BlitBitmapTiled(Bitmap *, Bitmap *, int, int, int, int, int, int, int,int);
-void FadeRectangle(Bitmap *bitmap, int, int, int, int, int, int, int,
+void FadeRectangle(int, int, int, int, int, int, int,
 		   void (*draw_border_function)(void));
 void FillRectangle(Bitmap *, int, int, int, int, Pixel);
 void ClearRectangle(Bitmap *, int, int, int, int);
 void ClearRectangleOnBackground(Bitmap *, int, int, int, int);
-void SetClipMask(Bitmap *, GC, Pixmap);
-void SetClipOrigin(Bitmap *, GC, int, int);
 void BlitBitmapMasked(Bitmap *, Bitmap *, int, int, int, int, int, int);
+boolean DrawingDeactivated(int, int, int, int);
 boolean DrawingOnBackground(int, int);
+boolean DrawingAreaChanged();
 void BlitBitmapOnBackground(Bitmap *, Bitmap *, int, int, int, int, int, int);
+void BlitTexture(Bitmap *, int, int, int, int, int, int);
+void BlitTextureMasked(Bitmap *, int, int, int, int, int, int);
+void BlitToScreen(Bitmap *, int, int, int, int, int, int);
+void BlitToScreenMasked(Bitmap *, int, int, int, int, int, int);
 void DrawSimpleBlackLine(Bitmap *, int, int, int, int);
 void DrawSimpleWhiteLine(Bitmap *, int, int, int, int);
 void DrawLines(Bitmap *, struct XY *, int, Pixel);
@@ -1276,21 +1490,22 @@ Pixel GetPixel(Bitmap *, int, int);
 Pixel GetPixelFromRGB(Bitmap *, unsigned int,unsigned int,unsigned int);
 Pixel GetPixelFromRGBcompact(Bitmap *, unsigned int);
 
-void FlushDisplay(void);
-void SyncDisplay(void);
 void KeyboardAutoRepeatOn(void);
 void KeyboardAutoRepeatOff(void);
-boolean PointerInWindow(DrawWindow *);
 boolean SetVideoMode(boolean);
+void SetVideoFrameDelay(unsigned int);
+unsigned int GetVideoFrameDelay();
 boolean ChangeVideoModeIfNeeded(boolean);
 
 Bitmap *LoadImage(char *);
 Bitmap *LoadCustomImage(char *);
 void ReloadCustomImage(Bitmap *, char *);
 
-Bitmap *ZoomBitmap(Bitmap *, int, int);
-void CreateBitmapWithSmallBitmaps(Bitmap *, int);
-void ScaleBitmap(Bitmap *, int);
+void ReCreateGameTileSizeBitmap(Bitmap **);
+void CreateBitmapWithSmallBitmaps(Bitmap **, int, int);
+void CreateBitmapTextures(Bitmap **);
+void FreeBitmapTextures(Bitmap **);
+void ScaleBitmap(Bitmap **, int);
 
 void SetMouseCursor(int);
 
@@ -1298,7 +1513,6 @@ void OpenAudio(void);
 void CloseAudio(void);
 void SetAudioMode(boolean);
 
-void InitEventFilter(EventFilter);
 boolean PendingEvent(void);
 void NextEvent(Event *event);
 void PeekEvent(Event *event);
@@ -1306,6 +1520,8 @@ Key GetEventKey(KeyEvent *, boolean);
 KeyMod HandleKeyModState(Key, int);
 KeyMod GetKeyModState();
 KeyMod GetKeyModStateFromEvents();
+void StartTextInput(int, int, int, int);
+void StopTextInput();
 boolean CheckCloseWindowEvent(ClientMessageEvent *);
 
 void InitJoysticks();
